/*   This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; version 2 of the License.
   
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     (C) 2006 schan
*/
#include "bitmapfont.h"
#include <iostream>

BitmapFont::BitmapFont()
{
	bitmap = NULL;
}

BitmapFont::~BitmapFont()
{
	close();
}

		
// inherited from Font
void BitmapFont::load(const std::string &filename, const std::wstring &chars)
{
	close();
	
	bitmap = SDL_LoadBMP(filename.c_str());

	SDL_Color separator_color = get_pixel(bitmap,0,0);
	SDL_Color new_color = separator_color;
	SDL_Color old_color = separator_color;
	bg_color = separator_color;

	int x=0;
	int charCount=0;
	wchar_t currentChar = L' ';
	
	while( x<bitmap->w )
	{
		// get current pixel's color
		new_color = get_pixel(bitmap,x,0);
		
		// look for the mask color (the first that is different from background color)
		if (compare_colors(bg_color,separator_color)) 
		{
			if (!compare_colors(new_color,separator_color))
			{
				bg_color = new_color;
				SDL_SetColorKey(bitmap,SDL_SRCCOLORKEY,SDL_MapRGB(bitmap->format,bg_color.r,bg_color.g,bg_color.b));
			}
		}
		
		if (!compare_colors(bg_color,separator_color))
		{				
			// detect glyph start
			if (compare_colors(old_color,separator_color) && compare_colors(new_color,bg_color)) 
			{
				currentChar = chars[charCount++];
				clip_rect[currentChar].x=x;
			}				
			
			// detect glyph end
			if (compare_colors(old_color,bg_color) && compare_colors(new_color,separator_color))
			{
				clip_rect[currentChar].y=0;
				clip_rect[currentChar].w=x-clip_rect[currentChar].x;
				clip_rect[currentChar].h=bitmap->h;						
			}
		}
		
		x++;
		old_color = new_color;
	}	
				
}

int BitmapFont::draw_unicode(const Uint16 *text, SDL_Surface *dest, int x, int y,  bool centerH, bool centerV)
{
	int w = get_width(text);
	int offsetX = 0;
	int offsetY = 0;
	if (centerV) offsetY = -bitmap->h>>1;
	if (centerH) offsetX = -w>>1;
	
	SDL_Rect destRect;					
	destRect.x = x + offsetX;
	int i=0;

	const Uint16 *ch;
	for(ch=text; *ch; ++ch)
	{
		destRect.y = y + offsetY; // restores destRect because SDL_BlitSurface modifies it

		SDL_BlitSurface(bitmap,&clip_rect[text[i]],dest,&destRect);
		destRect.x += clip_rect[text[i]].w;
		i++;
	}
	return w;
}

int BitmapFont::draw_unicode(Uint16 unicode_char, SDL_Surface *dest, int x, int y)
{
	SDL_Rect destRect;					
	destRect.x = x;
	destRect.y = y;
	destRect.w = clip_rect[unicode_char].w;
	destRect.h = clip_rect[unicode_char].h;
	SDL_BlitSurface(bitmap,&clip_rect[unicode_char],dest,&destRect);
	return destRect.w;
}


int BitmapFont::draw_text(const char *text, SDL_Surface *dest, int x, int y,  bool centerH, bool centerV)
{
	int w = get_width(text);
	int offsetX = 0;
	int offsetY = 0;
	if (centerV) offsetY = -bitmap->h>>1;
	if (centerH) offsetX = -w>>1;
	
	SDL_Rect destRect;					
	destRect.x = x + offsetX;
	int i=0;

	const char *ch;
	for(ch=text; *ch; ++ch)
	{
		destRect.y = y + offsetY; // restores destRect because SDL_BlitSurface modifies it

		SDL_BlitSurface(bitmap,&clip_rect[text[i]],dest,&destRect);
		destRect.x += clip_rect[text[i]].w;
		i++;
	}
	return w;
}

int BitmapFont::draw_char(char character, SDL_Surface *dest, int x, int y)
{
	SDL_Rect destRect;					
	destRect.x = x;
	destRect.y = y;
	destRect.w = clip_rect[character].w;
	destRect.h = clip_rect[character].h;
	SDL_BlitSurface(bitmap,&clip_rect[character],dest,&destRect);
	return destRect.w;
}

int BitmapFont::get_width(const char *text)
{
	int i=0;
	int w=0;

	const char *ch;
	for(ch=text; *ch; ++ch)
	{
		w += clip_rect[text[i++]].w;
	}		
	return w;
}

int BitmapFont::get_width(const Uint16 *text)
{
	int i=0;
	int w=0;

	const Uint16 *ch;
	for(ch=text; *ch; ++ch)
	{
		w += clip_rect[text[i++]].w;
	}		
	return w;
}

int BitmapFont::get_width(Uint16 unicode_char)
{
	return clip_rect[unicode_char].w;
}

int BitmapFont::get_height()
{
	if (bitmap==NULL) return 0;
	else return bitmap->h;
}

void BitmapFont::close()
{
	if (bitmap!=NULL)
	{
		SDL_FreeSurface(bitmap);
		bitmap = NULL;
		clip_rect.clear();
	}
}

SDL_Color BitmapFont::get_bg_color()
{
	return bg_color;
}

