/*   This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; version 2 of the License.
   
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     (C) 2006 schan
*/
#include "state_history.h"
#include "defines.h"
#include "bitmap.h"
#include "SDL/SDL.h"
#include "shared.h"
#include "util.h"
#include "gui.h"
#include "gui_skin.h"
#include "history.h"
#include "widget_text.h"
#include "widget_separator.h"
#include "widget_history.h"
#include <dirent.h>
#include <string>

#include <fstream>

#define HISTORY_FILE "path.cfg"

// AppState //////////////////////////////////////////////////////////////////////////////////

HistoryState::~HistoryState()
{
	delete [] status_text;
}

void HistoryState::on_init()
{	
	gui = new Gui(10,50,300,180);
		
	img_text = new Bitmap("pixmaps/text.bmp");
	img_bin = new Bitmap("pixmaps/binary.bmp");
	
	status_text = util::utf8_to_unicode("èAbrir   ïVolver al menú");
};
		
void HistoryState::on_button_down(int button)
{
	gui->on_button_down(button);
	redraw();
	
	if (button==GP2X_BUTTON_START)
	{
		app->change_to_state(ST_MENU);
	}

	else if (button==GP2X_BUTTON_A)
	{
		/*
		// change encoding
		redraw();		
		*/
	}
	else if (button==GP2X_BUTTON_B)
	{
		HistoryWidget *w = (HistoryWidget *)gui->get_selected_widget();
		shared::text->set_file(w->get_full_name(),w->get_encoding());
		app->change_to_state(ST_LOADING);
	}
};

void HistoryState::on_button_up(int button)
{
};
	
void HistoryState::think(bool first)
{
	if (first)
	{	
		refresh();
		redraw();
	}
}

void HistoryState::redraw()
{
	shared::background_image->draw(shared::screen,0,0);

	gui->draw(shared::screen);
	GuiSkin::get_small_font()->draw_unicode(status_text,shared::screen,160,228,true);
		
	SDL_Flip(shared::screen);
};


void HistoryState::refresh()
{
	gui->clear();
	gui->add_widget(new TextWidget("Historial", false, true));
	gui->add_widget(new SeparatorWidget(10));
	
	shared::history->load();
	for (int i=shared::history->get_item_count()-1; i>=0; i--)
	{
		// extract path 
		std::string fullfilename = shared::history->get_item_filename(i);
		int last_slash = fullfilename.find_last_of('/');
		std::string path = fullfilename.substr(0,last_slash);
		std::string filename = fullfilename.substr(last_slash+1,fullfilename.length()-last_slash-1);
		
		Bitmap *icon = (filename.length()>4 && filename.substr(filename.length()-4,4)==".txt" )?img_text:img_bin;
		
		gui->add_widget(new HistoryWidget(path, filename, icon, shared::history->get_item_percent(i) ) );
	}
}
