/*   This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; version 2 of the License.
   
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     (C) 2006 schan
*/
#include "state_selector.h"
#include "defines.h"
#include "bitmap.h"
#include "SDL/SDL.h"
#include "shared.h"
#include "util.h"
#include "gui.h"
#include "gui_skin.h"
#include "widget_text.h"
#include "widget_file.h"
#include <dirent.h>
#include <string>

#include <fstream>

#define PATH_FILE "path.cfg"

// AppState //////////////////////////////////////////////////////////////////////////////////

SelectorState::~SelectorState()
{
	delete [] status_text;
}

void SelectorState::on_init()
{
	std::ifstream file(PATH_FILE);
    file >> path;
	file.close();

	
	
	gui = new Gui(10,80,300,130);
	
	img_folder = new Bitmap("pixmaps/folder.bmp");
	img_text = new Bitmap("pixmaps/text.bmp");
	img_bin = new Bitmap("pixmaps/binary.bmp");
	
	filter = false;
	toggle_filter();
};
		
void SelectorState::on_button_down(int button)
{
	gui->on_button_down(button);
	redraw();
	
	if (button==GP2X_BUTTON_START)
	{
		app->change_to_state(ST_MENU);
	}

	if (button==GP2X_BUTTON_X)
	{
		toggle_filter(); // toggle_filter calls rescan()
		redraw();
	}
	else if (button==GP2X_BUTTON_A)
	{
		// change encoding
		redraw();		
	}
	else if (button==GP2X_BUTTON_B)
	{
		FileWidget *w = (FileWidget *)gui->get_selected_widget();
		std::cout << "Open " << w->get_full_name() << "\n";

		if (w->is_directory())
		{
			if (w->get_filename()=="..") // go up
			{
				int last_slash = path.find_last_of('/');
				path = path.substr(0,last_slash);
				if (path=="") path="/";				
			}
			else
			{
				if (path!="/") path +="/";
				path += w->get_filename();
			}
			
			
			rescan();
			redraw();
		}
		else
		{
			// save path
			std::ofstream file(PATH_FILE);
		    file << path << std::endl;
			file.close();			
			
			// load file		
			shared::text->set_file(w->get_full_name(),w->get_encoding());
			
			// add file to history
			shared::history->add(w->get_full_name(),0.0);
			
			app->change_to_state(ST_LOADING);
		}
	}
};

void SelectorState::on_button_up(int button)
{
};
	
void SelectorState::think(bool first)
{
	if (first)
	{	
		redraw();
	}
}

void SelectorState::redraw()
{
	shared::background_image->draw(shared::screen,0,0);

	GuiSkin::get_small_font()->draw_text(path.c_str(),shared::screen,10,60,false);
	gui->draw(shared::screen);
	GuiSkin::get_small_font()->draw_unicode(status_text,shared::screen,160,228,true);
		
	SDL_Flip(shared::screen);
};

void SelectorState::rescan()
{
	gui->clear();
	
	// unless we are in root folder, 
	// ALWAYS add the parent folder, so we can go back if we enter in a 
	// folder withouth read permission
	if (path.length()>1) { gui->add_widget(new FileWidget(path,"..", true, img_folder)); }
	

   struct dirent *dit;
	DIR *dirp;
	
	// loop 2 times
	
	// first for directories
	if ((dirp=opendir(path.c_str())) != NULL)
	{
		while ((dit = readdir(dirp)) != NULL)
		{
			if (dit->d_type==DT_DIR)
			{
				if ( strcmp(dit->d_name,".")==0 || strcmp(dit->d_name,"..")==0 )
				{
					// dont show
				}
				else
				{
					gui->add_widget(new FileWidget(path,dit->d_name, true, img_folder));				
				}
			}
			
		}
	}
	// then for files
	if ((dirp=opendir(path.c_str())) != NULL)
	{
		while ((dit = readdir(dirp)) != NULL)
		{
			if (dit->d_type!=DT_DIR)
			{
				int len = strlen(dit->d_name);
				if (len>3 && dit->d_name[len-4]=='.' && dit->d_name[len-3]=='t' 
				          && dit->d_name[len-2]=='x' && dit->d_name[len-1]=='t')
				{
					gui->add_widget(new FileWidget(path.c_str(),dit->d_name, false, img_text));
				}
				else if (!filter)
				{
					gui->add_widget(new FileWidget(path.c_str(),dit->d_name, false, img_bin));
				}
			}
		}
	}
}

void SelectorState::toggle_filter()
{
	filter = !filter;
	if (filter)
	{
		status_text = util::utf8_to_unicode("èAbrir   ïVolver al menú   ìMostrar todos");
	}
	else
	{
		status_text = util::utf8_to_unicode("èAbrir   ïVolver al menú   ìMostrar solo txt");
	}
	rescan();
}
